package com.simuwang.manage.service.competition;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import com.simuwang.base.common.conts.DateConst;
import com.simuwang.base.mapper.*;
import com.simuwang.base.pojo.dos.*;
import com.simuwang.base.pojo.dto.FundNavDataDTO;
import com.simuwang.base.pojo.dto.FundNavDeletionDTO;
import com.simuwang.base.pojo.dto.competition.CompetitionBaseResultDTO;
import com.simuwang.base.pojo.dto.competition.ZhzqCompetitionResultDTO;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class FundNavServiceImpl implements FundNavService {

    private final NavMapper navMapper;
    private final FundInfoMapper fundInfoMapper;
    private final AssetMapper assetMapper;
    private final DeletionInfoMapper deletionInfoMapper;
    private final TradeDateMapper tradeDateMapper;
    private final CompetitionFactory competitionFactory;
    private final RcCompetitionTaskDateMapper competitionTaskDateMapper;
    private final RcCompetitionCalcDateMapper competitionCalcDateMapper;

    public FundNavServiceImpl(NavMapper navMapper, FundInfoMapper fundInfoMapper,
                              AssetMapper assetMapper, DeletionInfoMapper deletionInfoMapper,
                              TradeDateMapper tradeDateMapper, CompetitionFactory competitionFactory,
                              RcCompetitionTaskDateMapper competitionTaskDateMapper, RcCompetitionCalcDateMapper competitionCalcDateMapper) {
        this.navMapper = navMapper;
        this.fundInfoMapper = fundInfoMapper;
        this.assetMapper = assetMapper;
        this.deletionInfoMapper = deletionInfoMapper;
        this.tradeDateMapper = tradeDateMapper;
        this.competitionFactory = competitionFactory;
        this.competitionTaskDateMapper = competitionTaskDateMapper;
        this.competitionCalcDateMapper = competitionCalcDateMapper;
    }

    @Override
    public List<FundNavDataDTO> getFungNavData(Integer competitionId) {
        List<FundNavDataDTO> navDataDTOList = CollUtil.newArrayList();
        String date = DateUtil.format(new Date(), DateConst.YYYY_MM_DD);
        List<RcCompetitionTaskDateDO> taskDateDOList = competitionTaskDateMapper.queryByNavDetailDate(date, competitionId);
        if (CollUtil.isEmpty(taskDateDOList)) {
            return navDataDTOList;
        }
        List<String> periodList = taskDateDOList.stream().map(RcCompetitionTaskDateDO::getPriod).distinct().toList();
        List<RcCompetitionCalcDateDO> calcDateDOList = competitionCalcDateMapper.queryByPriod(periodList, competitionId);
        for (RcCompetitionCalcDateDO calcDateDO : calcDateDOList) {
            String period = calcDateDO.getPriod();
            String beginDate = calcDateDO.getBeginDate();
            String endDate = calcDateDO.getEndDate();
            List<NavDO> navDOList = navMapper.queryAllNav(beginDate, endDate);
            if (CollUtil.isEmpty(navDOList)) {
                continue;
            }
            // 过滤出周频的净值点
            List<TradeDateDO> tradeDateDOList = tradeDateMapper.selectTradeDate(beginDate, endDate);
            navDOList = filterWeekFrequencyNav(navDOList, tradeDateDOList);

            List<String> fundIdList = navDOList.stream().map(NavDO::getFundId).distinct().toList();
            List<FundAndCompanyInfoDO> fundInfoCompanyNameList = fundInfoMapper.queryFundAndTrustByFundId(fundIdList);
            Map<String, FundAndCompanyInfoDO> fundIdCompanyNameMap = fundInfoCompanyNameList.stream().collect(Collectors.toMap(FundAndCompanyInfoDO::getFundId, v -> v));
            List<AssetDO> assetDOList = assetMapper.queryAssetByFundId(fundIdList);
            Map<String, List<AssetDO>> fundIdAssetMap = MapUtil.newHashMap();
            if (CollUtil.isNotEmpty(assetDOList)) {
                fundIdAssetMap = assetDOList.stream().collect(Collectors.groupingBy(AssetDO::getFundId));
            }

            Map<String, List<NavDO>> fundIdNavMap = navDOList.stream().collect(Collectors.groupingBy(NavDO::getFundId));
            for (Map.Entry<String, List<NavDO>> fundIdNavEntry : fundIdNavMap.entrySet()) {
                String fundId = fundIdNavEntry.getKey();
                List<NavDO> fundNavDoList = fundIdNavEntry.getValue();
                List<AssetDO> fundAssetDoList = fundIdAssetMap.get(fundId);
                FundAndCompanyInfoDO fundInfoDO = fundIdCompanyNameMap.get(fundId);
                List<FundNavDataDTO> fundNavDataDTOList = buildFundNavDataDTO(fundInfoDO, fundNavDoList, fundAssetDoList);
                fundNavDataDTOList.forEach(e -> e.setPriod(period));
                navDataDTOList.addAll(fundNavDataDTOList);
            }
        }
        return navDataDTOList;
    }

    @Override
    public List<FundNavDeletionDTO> getFundNavDeletion(Integer competitionId) {
        List<FundNavDeletionDTO> navDeletionDTOList = CollUtil.newArrayList();
        String date = DateUtil.format(new Date(), DateConst.YYYY_MM_DD);
        List<RcCompetitionTaskDateDO> taskDateDOList = competitionTaskDateMapper.queryByNavDeletionDate(date, competitionId);
        if (CollUtil.isEmpty(taskDateDOList)) {
            return navDeletionDTOList;
        }
        List<String> periodList = taskDateDOList.stream().map(RcCompetitionTaskDateDO::getPriod).distinct().toList();
        List<RcCompetitionCalcDateDO> calcDateDOList = competitionCalcDateMapper.queryByPriod(periodList, competitionId);
        for (RcCompetitionCalcDateDO calcDateDO : calcDateDOList) {
            String period = calcDateDO.getPriod();
            String beginDate = calcDateDO.getBeginDate();
            String endDate = calcDateDO.getEndDate();
            // 缺失类型-1对应全部，1-净值缺失，2-对应规模缺失，3-对应分红缺失,5-估值缺失
            List<FundDeletionInfoDO> deletionInfoDOList = deletionInfoMapper.getFundNavDeletion(1, beginDate, endDate);
            if (CollUtil.isEmpty(deletionInfoDOList)) {
                continue;
            }
            List<String> fundIdList = deletionInfoDOList.stream().map(FundDeletionInfoDO::getFundId).distinct().toList();
            List<FundAndCompanyInfoDO> fundInfoCompanyNameList = fundInfoMapper.queryFundAndTrustByFundId(fundIdList);
            Map<String, FundAndCompanyInfoDO> fundIdCompanyNameMap = fundInfoCompanyNameList.stream().collect(Collectors.toMap(FundAndCompanyInfoDO::getFundId, v -> v));
            // 获取净值日期所在年和所在周
            List<String> dateList = deletionInfoDOList.stream().map(FundDeletionInfoDO::getDeletionDate).distinct().toList();
            Map<String, TradeDateDO> tradeDateDoMap = getYearAndWeekOfDate(dateList);
            Map<String, List<FundDeletionInfoDO>> fundIdDeletionInfoMap = deletionInfoDOList.stream().collect(Collectors.groupingBy(FundDeletionInfoDO::getFundId));
            for (Map.Entry<String, List<FundDeletionInfoDO>> fundIdDeletionInfoEntry : fundIdDeletionInfoMap.entrySet()) {
                String fundId = fundIdDeletionInfoEntry.getKey();
                List<FundDeletionInfoDO> fundDeletionInfoDOList = fundIdDeletionInfoEntry.getValue();
                FundAndCompanyInfoDO fundAndCompanyInfoDO = fundIdCompanyNameMap.get(fundId);
                List<FundNavDeletionDTO> fundNavDataDTOList = buildFundNavDeletionDTO(fundAndCompanyInfoDO, fundDeletionInfoDOList, tradeDateDoMap);
                fundNavDataDTOList.forEach(e -> e.setPriod(period));
                navDeletionDTOList.addAll(fundNavDataDTOList);
            }
        }
        return navDeletionDTOList;
    }

    @Override
    public List<? extends CompetitionBaseResultDTO> getCompetitionResult(Integer competitionId) {
        List<ZhzqCompetitionResultDTO> dataList = CollUtil.newArrayList();
        AbstractCompetitionResultService competitionResult = competitionFactory.getInstance(competitionId);
        if (competitionResult == null) {
            return dataList;
        }
        return competitionResult.getCompetitionResult();
    }

    private List<FundNavDeletionDTO> buildFundNavDeletionDTO(FundAndCompanyInfoDO fundAndCompanyInfoDO, List<FundDeletionInfoDO> fundDeletionInfoDOList, Map<String, TradeDateDO> tradeDateDoMap) {
        String fundName = fundAndCompanyInfoDO.getFundName();
        String trustName = fundAndCompanyInfoDO.getCompanyName();
        String trustRegisterNumber = fundAndCompanyInfoDO.getCompanyRegisterNumber();
        String registerNumber = fundAndCompanyInfoDO.getRegisterNumber();
        return fundDeletionInfoDOList.stream().map(e -> {
            FundNavDeletionDTO fundNavDeletionDTO = new FundNavDeletionDTO();
            fundNavDeletionDTO.setTrustName(trustName);
            fundNavDeletionDTO.setTrustRegisterNumber(trustRegisterNumber);
            fundNavDeletionDTO.setFundId(e.getFundId());
            fundNavDeletionDTO.setFundName(fundName);
            fundNavDeletionDTO.setRegisterNumber(registerNumber);
            TradeDateDO tradeDateDO = tradeDateDoMap.get(e.getDeletionDate());
            fundNavDeletionDTO.setYear(tradeDateDO != null ? tradeDateDO.getEndYear() : null);
            fundNavDeletionDTO.setWeek(tradeDateDO != null ? String.valueOf(tradeDateDO.getWeekOfYear()) : null);
            fundNavDeletionDTO.setDate(e.getDeletionDate());
            return fundNavDeletionDTO;
        }).sorted(Comparator.comparing(FundNavDeletionDTO::getFundName).thenComparing(FundNavDeletionDTO::getDate)).toList();
    }

    private List<FundNavDataDTO> buildFundNavDataDTO(FundAndCompanyInfoDO fundInfoDO, List<NavDO> fundNavDoList, List<AssetDO> fundAssetDoList) {
        String fundName = fundInfoDO.getFundName();
        String registerNumber = fundInfoDO.getRegisterNumber();
        String trustName = fundInfoDO.getCompanyName();
        String trustRegisterNumber = fundInfoDO.getCompanyRegisterNumber();
        Map<String, BigDecimal> priceDateAssetMap = MapUtil.newHashMap();
        if (CollUtil.isNotEmpty(fundAssetDoList)) {
            priceDateAssetMap = fundAssetDoList.stream().collect(Collectors.toMap(k -> DateUtil.format(k.getPriceDate(), DateConst.YYYY_MM_DD), AssetDO::getAssetNet));
        }
        Map<String, BigDecimal> finalPriceDateAssetMap = priceDateAssetMap;
        return fundNavDoList.stream().map(e -> {
            FundNavDataDTO fundNavDataDTO = new FundNavDataDTO();
            fundNavDataDTO.setTrustName(trustName);
            fundNavDataDTO.setTrustRegisterNumber(trustRegisterNumber);
            fundNavDataDTO.setFundId(e.getFundId());
            fundNavDataDTO.setFundName(fundName);
            fundNavDataDTO.setRegisterNumber(registerNumber);
            String priceDate = e.getPriceDate() != null ? DateUtil.format(e.getPriceDate(), DateConst.YYYY_MM_DD) : null;
            fundNavDataDTO.setPriceDate(priceDate);
            fundNavDataDTO.setNav(e.getNav() != null ? String.valueOf(e.getNav()) : null);
            fundNavDataDTO.setCumulativeNavWithdrawal(e.getCumulativeNavWithdrawal() != null ? String.valueOf(e.getCumulativeNavWithdrawal()) : null);
            BigDecimal asset = finalPriceDateAssetMap.get(priceDate) != null ? finalPriceDateAssetMap.get(priceDate).setScale(2, RoundingMode.HALF_UP) : null;
            fundNavDataDTO.setAsset(asset != null ? String.valueOf(asset) : null);
            return fundNavDataDTO;
        }).sorted(Comparator.comparing(FundNavDataDTO::getFundName).thenComparing(FundNavDataDTO::getPriceDate)).toList();
    }

    private Map<String, TradeDateDO> getYearAndWeekOfDate(List<String> dateList) {
        Map<String, TradeDateDO> tradeDateDoMap = MapUtil.newHashMap();
        if (CollUtil.isEmpty(dateList)) {
            return tradeDateDoMap;
        }
        List<TradeDateDO> tradeDateDOList = CollUtil.newArrayList();
        List<List<String>> partition = ListUtil.partition(dateList, 500);
        partition.forEach(e -> tradeDateDOList.addAll(tradeDateMapper.queryByDate(dateList)));
        if (CollUtil.isNotEmpty(tradeDateDOList)) {
            tradeDateDoMap = tradeDateDOList.stream().collect(Collectors.toMap(k -> DateUtil.format(k.getTradeDate(), DateConst.YYYY_MM_DD), v -> v));
        }
        return tradeDateDoMap;
    }

    private List<NavDO> filterWeekFrequencyNav(List<NavDO> navDOList, List<TradeDateDO> tradeDateDOList) {
        List<NavDO> dataList = CollUtil.newArrayList();
        if(CollUtil.isEmpty(navDOList)){
          return dataList;
        }
        Map<Integer, List<TradeDateDO>> yearWeekTradeDateMap = tradeDateDOList.stream().collect(Collectors.groupingBy(TradeDateDO::getYearWeek));

        Map<String, List<NavDO>> fundIdNavDoMap = navDOList.stream().collect(Collectors.groupingBy(NavDO::getFundId));
        for (Map.Entry<String, List<NavDO>> fundIdNavDoEntry : fundIdNavDoMap.entrySet()) {
            List<NavDO> navDoList = fundIdNavDoEntry.getValue();
            for (Map.Entry<Integer, List<TradeDateDO>> yearWeekTradeDateEntry : yearWeekTradeDateMap.entrySet()) {
                List<Date> tradeDateList = yearWeekTradeDateEntry.getValue().stream().map(TradeDateDO::getTradeDate).toList();
                List<NavDO> collect = navDoList.stream().filter(e -> tradeDateList.contains(e.getPriceDate())).toList();
                if(CollUtil.isEmpty(collect)){
                    continue;
                }
                NavDO navDO = collect.stream().max(Comparator.comparing(NavDO::getPriceDate)).orElse(null);
                dataList.add(navDO);
            }
        }
        return dataList;
    }

}
